// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//	DOOM graphics stuff for GGI ( http://synergy.caltech.edu/~ggi/ )
//	Maintainer: Uwe Maurer - uwe_maurer@t-online.de
//			http://home.t-online.de/home/uwe_maurer
//-----------------------------------------------------------------------------

static const char
rcsid[] = "$Id: i_x.c,v 1.6 1997/02/03 22:45:10 b1 Exp $";

#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <ggi/libggi.h>

#include "doomstat.h"
#include "i_system.h"
#include "v_video.h"
#include "m_argv.h"
#include "d_main.h"
#include "r_draw.h"

#include "doomdef.h"

ggi_visual_t 	ggivis;

int		width;
int		height;

int key(int sym)
{
    	int rc=KVAL(U(sym));
	
	switch(KTYP(U(sym)))
	{
		case KT_LATIN:
		case KT_LETTER:
		case KT_META:
		switch(KVAL(U(sym)))
		{
			case '\033':	rc = KEY_ESCAPE;	break;
			case '\011':	rc = KEY_TAB;		break;
			
			default:
				if (sym >='A'  && sym <='Z')
					rc = rc - 'A' + 'a';
		}
		break;
		default:
		switch(U(sym))
		{
			 
      			case K_UP:	rc = KEY_UPARROW;	break;
      			case K_DOWN:	rc = KEY_DOWNARROW;	break;
      			case K_LEFT:	rc = KEY_LEFTARROW;	break;
			case K_RIGHT:	rc = KEY_RIGHTARROW;	break;
			case K_ENTER:	rc = KEY_ENTER;		break;
			case K_NORMAL_CTRL: rc=KEY_RCTRL; 	break;
			case K_NORMAL_SHIFT: rc=KEY_RSHIFT;	break;

			case K_NORMAL_ALTGR:
			case K_NORMAL_ALT: rc=KEY_RALT;		break;
      			case K_F1:	rc = KEY_F1;		break;
			case K_F2:	rc = KEY_F2;		break;
			case K_F3:	rc = KEY_F3;		break;
			case K_F4:	rc = KEY_F4;		break;
			case K_F5:	rc = KEY_F5;		break;
			case K_F6:	rc = KEY_F6;		break;
			case K_F7:	rc = KEY_F7;		break;
			case K_F8:	rc = KEY_F8;		break;
			case K_F9:	rc = KEY_F9;		break;
			case K_F10:	rc = KEY_F10;		break;
			case K_F11:	rc = KEY_F11;		break;
			case K_F12:	rc = KEY_F12;		break;
      			case K_PAUSE:	rc = KEY_PAUSE;		break;
	
		}
	}

    return rc;

}

void I_ShutdownGraphics(void)
{
	ggiClose(ggivis);
	ggiExit();
}


void I_StartFrame (void)
{

}

void I_GetEvent(void)
{
	event_t event;
	ggi_event ev;
	int evmask=emKeyPress|emKeyRelease;

	while (1)
	{
		struct timeval t={0,0};
		if (ggiEventPoll(ggivis,evmask,&t)==0) return;

		ggiEventRead(ggivis,&ev,evmask);

		switch (ev.any.type)
		{
			case evKeyPress:
				event.type = ev_keydown;
				event.data1 = key(ev.key.sym);
				D_PostEvent(&event);
				break;
		 	case evKeyRelease:
				event.type = ev_keyup;
				event.data1 = key(ev.key.sym);
				D_PostEvent(&event);
				break;
	    }
	}
}

void I_StartTic (void)
{
	I_GetEvent();
}


void I_UpdateNoBlit (void)
{
}

void I_FinishUpdate (void)
{

    static int	lasttic;
    int		tics;
    int		i;

    // draws little dots on the bottom of the screen
    if (devparm)
    {
	i = I_GetTime();
	tics = i - lasttic;
	lasttic = i;
	if (tics > 20) tics = 20;

	for (i=0 ; i<tics*2 ; i+=2)
	    screens[0][ (SCREENHEIGHT-1)*SCREENWIDTH + i] = 0xff;
	for ( ; i<20*2 ; i+=2)
	    screens[0][ (SCREENHEIGHT-1)*SCREENWIDTH + i] = 0x0;
    
    }
	ggiPutBox(ggivis,0,0,width,height,screens[0]);
	ggiFlush(ggivis);
	I_GetEvent();
}


void I_ReadScreen (byte* scr)
{
    memcpy (scr, screens[0], SCREENWIDTH*SCREENHEIGHT);
}

void I_SetPalette (byte* pal)
{
	int i;
	ggi_color col[256];
	for (i=0;i<256;i++) 
	{
		col[i].r=(*pal++) << (GGI_COLOR_PRECISION-8);
		col[i].g=(*pal++) << (GGI_COLOR_PRECISION-8);
		col[i].b=(*pal++) << (GGI_COLOR_PRECISION-8);
	}
	ggiSetPaletteVec(ggivis,0,256,col);
}

void I_InitGraphics(void)
{
	ggiInit();
	ggivis=ggiOpen(NULL);

	signal(SIGINT, (void (*)(int)) I_Quit);

	width = SCREENWIDTH;
	height = SCREENHEIGHT;

	if (ggiSetGraphMode(ggivis,width,height,width,height,GT_8BIT)<0)
		I_Error("Can't set mode %ix%i \n",width,height);

	ggiSetInfoFlags(ggivis,GGIFLAG_ASYNC);

}

